@@ -1,7 +1,8 @@
 #![feature(allow_internal_unstable)]
-#![feature(const_fn)]
+#![feature(bench_black_box)]
 #![feature(const_panic)]
 #![feature(extend_one)]
+#![feature(iter_zip)]
 #![feature(unboxed_closures)]
 #![feature(test)]
 #![feature(fn_traits)]
